<?php
/**
 * @file
 * This file is part of the Twizo Drupal 7 2fa extension.
 *
 * (c) Twizo <info@twizo.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * File that was distributed with this source code.
 */

/**
 * Function that communicates with the Twizo API.
 * @param string $command
 * @param array $data
 * @return bool|\Twizo\Api\Entity\Sms[]
 */
function twizo_command($command = '', $data = array()){
    require_once(DRUPAL_ROOT . '/vendor/twizo/lib-api-php/autoload.php');

    // Set account variables
    $apiKey = variable_get('twizo_api_key', FALSE);
    $apiServer = variable_get('twizo_api_server', FALSE);

    if(isset($data['uid'])){
        $uid = $data['uid'];
    } elseif (isset($data['name']) && isset($data['pass'])){
        $uid = user_authenticate($data['name'], $data['pass']);
    } else{
        $uid = NULL;
    }

    $account = isset($uid) ? user_load($uid) : NULL;
    $preferredMethod = isset($account->twizo_user['preferredmethod']) ? $account->twizo_user['preferredmethod'] : 'sms';
    $identifier = isset($account->twizo_user['identifier']) ? $account->twizo_user['identifier'] : NULL;
    $accountNumber = isset($account->twizo_user['number']) ? $account->twizo_user['number'] : NULL;
    $number = isset($data['number']) ? $data['number'] : $accountNumber;

    // Do nothing if account credentials aren't entered.
    if (!$apiKey || !$apiServer) {
        $errormsg = 'Please enter your API key and/or API server';
        drupal_set_message(t($errormsg), 'error');
        return FALSE;
    }

    $twizo = \Twizo\Api\Twizo::getInstance($apiKey, $apiServer);

    switch ($command) {
        case "sendSms":
            $sms = $twizo->createSms($data['message'], $data['recipient'], $data['sender']);
            $result = $sms->send();
            break;
        case "sendSimple":
            $sms = $twizo->createSms($data['message'], $data['recipient'], $data['sender']);
            $result = $sms->sendSimple();
            break;
        case "setTrusted":
            // Create a random token
            $token = bin2hex(drupal_random_bytes(30));
            // Create a hash with the user identifier and the above generated token.
            $hashedVersion = hash('sha512', sprintf('%s_%s', $identifier, $token));
            $data = array(
                'number' => $account->twizo_user['number'],
                'cookiehash' => $hashedVersion,
            );

            // Saves the $hashedVersion to the user database.
            twizo_user_update_cookiehash($data);

            // Set a cookie with the generated $token.
            $result = twizo_set_cookie($token);
            break;
        case "widgetSession":
            // Creates a widget session.
            if (isset($identifier) && variable_get('twizo_enable_backupcode') == 1) {
                $widgetSession = $twizo->createWidgetSession(NULL, $number);
                $widgetSession->setBackupCodeIdentifier($identifier);
            } else{
                $widgetSession = $twizo->createWidgetSession(NULL, $number);
            }
            $widgetSession->setPreferredType($preferredMethod);
            // If a sender has been set, change the sender to the sender set by the admin in the admin settings.
            if(variable_get('twizo_sender')){
                $widgetSession->setSender(variable_get('twizo_sender'));
            }
            $widgetSession->setTag('drupal 7');
            $widgetSession->create();
            $sessionToken = $widgetSession->getSessionToken();

            // Adds the required parameters to a js required for the widget to be functional.
            drupal_add_js(array('twizo' => array(
                'sessionToken' => $sessionToken,
                'uid' => $data['uid'],
                'name' => $data['name'],
                'pass' => $data['pass'],
                'number' => $data['number'],
                'widgetLogo' => variable_get('widget_logo'))), array('type' => 'setting')

            );
            $result = $sessionToken;
            break;
        case "checkApiKey":
            // Checks if a the API key is correct, if it's not correct set a drupal message with the Twizo Exception error.
            try {
                $twizo->getBalance();
                $result = TRUE;
            } catch (\Twizo\Api\Entity\Exception $e){
                drupal_set_message($e->getMessage(), 'error');
                $result = FALSE;
            }
            break;
        case "getWidgetStatus":
            // Returns the widget status returned by the Twizo API.
            if (variable_get('twizo_enable_backup_codes') == 1 && isset($identifier)) {
                $result = $twizo->getWidgetSession($data['sessionToken'], $data['number'], $identifier)->getStatus();
            } else {
                $result = $twizo->getWidgetSession($data['sessionToken'], $data['number'])->getStatus();
            }
            break;
        case "createBackupCode":
            // Creates backup codes with the Twizo API.
            $backupCode = $twizo->createBackupCode($identifier);
            $backupCode->create();
            $data = array(
                'number' => $number,
                'codesgenerated' => 1,
            );
            // Set the codesgenerated status in the user database. The module now knows that the codes have been generated at some point.
            twizo_user_update_codesgenerated($data);
            $result = $backupCode->getCodes();
            break;
        case "updateBackupCode":
            // Updates the backupcodes.
            $backupCode = $twizo->createBackupCode($identifier);
            $backupCode->update();
            $result = $backupCode->getCodes();
            break;
        case "getRemainingBackupCode":
            // Returns the number of backupcodes remaining in the Drupal API.
            $backupCode = $twizo->getBackupCode($identifier);
            $result = $backupCode->getAmountOfCodesLeft();
            break;
        case "getVerificationTypes":
            try {
                $result = $twizo->getVerificationTypes()->getVerificationTypes();
            }catch (\Twizo\Api\Exception $e){
                $result = drupal_set_message($e->getMessage());

            }
            break;
        default:
            $result = drupal_set_message(t('Unknown command: '. $command .' found in twizo_command()'), 'error');
            break;
    }
    return $result;
}

/**
 * Method for sending a test sms.
 * @param $message
 * @param $recipient
 * @param null $sender
 * @return bool
 */
function twizo_send($message, $recipient, $sender = null){
    if ($sender == null) {
        $sender = 'Twizo';
    }

    // Puts the parameters in an array.
    $data = array(
        'message' => $message,
        'recipient' => $recipient,
        'sender' => $sender
    );

    try {
        if (strlen($message < 160)) {
            twizo_command('sendSimple', $data);
        } else {
            twizo_command('sendSms', $data);
        }
        $response = TRUE;
    } catch (\Twizo\Api\Entity\Exception $e){
        drupal_set_message($e->getMessage(), 'error');
        $response = FALSE;
    }

    return $response;
}

